import org.json.JSONArray;
import org.json.JSONObject;

import java.nio.file.Files;
import java.nio.file.Path;

import static spark.Spark.*;

public class TodoApp {
    //This is a pretty butchered Java implementation, but I really don't want to write all that code...
    public static void main(String[] args) {
        port(8080);

        JSONArray todoList = generateTodoList();

        staticFiles.externalLocation("client/public/");
        Path filePath = Path.of("client/todo.html");

        try {
            String html = Files.readString(filePath);
            get("/", (req, res) -> html);
        } catch (Exception e) {
            System.out.println(e);
        }


        /**
        * get todo handler returns all the todos in the todoList
        * @return {json} json list with all the requested todos
        *
        * The client code imples a contract:
        * calling the server with GET on /todo should return all todos
        */
        get("/todo", (req, res) -> {
            res.type("application/json");
            return todoList.toString();
        });

        /**
        * post todo handler adds a todo to the list
        * @return {json} todo object with id and todo
        * The client code imples a contract:
        * calling the server with POST on /todo should add the incoming todo to our todoList and return the object with an id
        * {id: <id>, todo: <todotext>}
        */
        post("/todo", (req, res) -> {
            res.type("application/json");
            JSONObject requestBody = new JSONObject(req.body());
            int nextId = 0;

            for (int i = 0; i < todoList.length(); i++) {
                int currentId = todoList.getJSONObject(i).getInt("id");
                if (currentId >= nextId) {
                    nextId = currentId + 1;
                }
            }

            JSONObject newTodo = new JSONObject();
            newTodo.put("id", nextId);
            newTodo.put("todo", requestBody.getString("todo"));
            todoList.put(newTodo);
            return newTodo.toString();
        });

        /**
        * Delete todo handler deletes the todo with a specific id (/todo/:id)
        * @return {status} returns statuscode 204 (no content) to confirm
        * The client code imples a contract:
        * calling the server with DELETE on /todo should remove the todo with the provided id.
        */
        delete("/todo/:id", (req, res) -> {
            int idToDelete = Integer.parseInt(req.params("id"));
            for (int i = 0; i < todoList.length(); i++) {
                if(todoList.getJSONObject(i).getInt("id") == idToDelete) {
                    todoList.remove(i);
                    res.status(204);
                    return "";
                }
            }
            res.status(404);
            return "Todo not found";
        });
    }

    private static JSONArray generateTodoList(){
        JSONArray todoList = new JSONArray();
        String[] todoStrings = {"Add client code to your example","Make sure the CSS and JavaScript is in the correct folder","Run your server to verify the static files","Implement the GET todoHandler","Implement the POST todoHandler","Implement the DELETE todoHandler"};
        for (int i = 0; i < todoStrings.length; i++) {
            JSONObject todo = new JSONObject();
            todo.put("todo", todoStrings[i]);
            todo.put("id", i);
            todoList.put(todo);
        }
        return todoList;
    }
}